from _typeshed import Incomplete

def SeriesFactory(
    values,
    xvalues: Incomplete | None = ...,
    zvalues: Incomplete | None = ...,
    title: Incomplete | None = ...,
    title_from_data: bool = ...,
): ...
